#include <async.h>
#include <arpc.h>
#include "fadd.h"

void connected(int pfd, int cfd);
void accept_client(ptr<aclnt> ac, int fd);
void dispatch(ptr<aclnt> ac, ptr<asrv> s, svccb *sbp);
void replycb(ref<fadd_res> resp, svccb *sbp, clnt_stat stat);

int
main(int ac, char **av)
{
    if (ac != 3)
	fatal << "Usage: " << av[0] << " proxy-port connect-to-port\n";

    int pfd = inetsocket(SOCK_STREAM, atoi(av[1]));
    make_async(pfd);
    listen(pfd, 5);

    tcpconnect("localhost", atoi(av[2]), wrap(&connected, pfd));
    amain();
}

void
connected(int pfd, int cfd)
{
    if (cfd < 0)
	fatal << "cannot connect\n";

    ptr<axprt> x = axprt_stream::alloc(cfd);
    ptr<aclnt> ac = aclnt::alloc(x, fadd_prog_1);
    fdcb(pfd, selread, wrap(&accept_client, ac, pfd));
}

void
accept_client(ptr<aclnt> ac, int fd)
{
    sockaddr_in sin;
    socklen_t sinlen = sizeof(sin);
    int c = accept(fd, (sockaddr *) &sin, &sinlen);

    ptr<axprt> x = axprt_stream::alloc(c);
    ptr<asrv> s = asrv::alloc(x, fadd_prog_1);
    s->setcb(wrap(&dispatch, ac, s));
}

void
dispatch(ptr<aclnt> c, ptr<asrv> s, svccb *sbp)
{
    if (!sbp) {		/* EOF from client */
	s->setcb(0);
	return;
    }

    switch (sbp->proc()) {
    case FADDPROC_NULL:
	sbp->reply(0);
	return;

    case FADDPROC_FADD: {
	fadd_arg *arg = sbp->Xtmpl getarg<fadd_arg>();
	ref<fadd_res> resp = New refcounted<fadd_res>();

	c->call(FADDPROC_FADD, arg, resp, wrap(&replycb, resp, sbp));
	return;
    }

    default:
	sbp->reject(PROC_UNAVAIL);
    }
}

void
replycb(ref<fadd_res> resp, svccb *sbp, clnt_stat stat)
{
    if (stat)
	sbp->reject(SYSTEM_ERR);
    else
	sbp->reply(resp);
}
