#include <async.h>
#include <qhash.h>
#include <arpc.h>
#include "fadd.h"

static qhash<str, int> table;

void accept_client(int fd);
void dispatch(ptr<asrv> s, svccb *sbp);

int
main(int ac, char **av)
{
    if (ac != 2)
	fatal << "Usage: " << av[0] << " port\n";

    int fd = inetsocket(SOCK_STREAM, atoi(av[1]));
    make_async(fd);
    listen(fd, 5);
    fdcb(fd, selread, wrap(&accept_client, fd));

    table.insert("x", 5);
    table.insert("y", 0);

    amain();
}

void
accept_client(int fd)
{
    sockaddr_in sin;
    socklen_t sinlen = sizeof(sin);
    int c = accept(fd, (sockaddr *) &sin, &sinlen);

    ptr<axprt> x = axprt_stream::alloc(c);
    ptr<asrv> s = asrv::alloc(x, fadd_prog_1);
    s->setcb(wrap(&dispatch, s));
}

void
dispatch(ptr<asrv> s, svccb *sbp)
{
    if (!sbp) {		/* EOF from client */
	s->setcb(0);
	return;
    }

    switch (sbp->proc()) {
    case FADDPROC_NULL:
	sbp->reply(0);
	return;

    case FADDPROC_FADD: {
	fadd_arg *arg = sbp->Xtmpl getarg<fadd_arg>();
	fadd_res res;

	int *valp = table[arg->var];
	if (valp) {
	    *valp += arg->inc;
	    res.set_ok(true);
	    *res.sum = *valp;
	} else {
	    res.set_ok(false);
	}

	sbp->reply(&res);
	return;
    }

    default:
	sbp->reject(PROC_UNAVAIL);
    }
}
